import {Schema} from '../schema'
import ObjectId from '../object-id'
import Complex from '../complex'
import toMongooseSchema from '../mongoose-output'
import toTypescriptSchema from '../ts-output'
import toJSONSchema from '../json-schema-output'
import now from '../now'
import {R2Author} from './r2-common'

const schema: Schema = {
  fields: {
    _id: ObjectId,
    status: {
      type: String,
      mongooseDefault: 'inactive',
      enum: ['inactive', 'activating', 'active', 'private', 'expired', 'deleted', 'draft']
    },
    applicationForm: {type: ObjectId, mongooseRef: 'ApplicationForm', optional: true},
    applicationFormHistory: [{type: ObjectId, mongooseRef: 'ApplicationForm', optional: true}],
    pastApplicationFormsWithApplicants: [{type: ObjectId, mongooseRef: 'ApplicationForm', optional: true}],
    title: String,
    molTitle: String,
    firstApplicationDate: Date,
    lastApplicationDate: Date,
    createdBy: R2Author,
    contact: R2Author,
    jobPostingUrl: String,
    carRequired: Boolean,
    useCustomApplicationUrl: Boolean,
    updateProfileJobPosting: Boolean,
    o2FormType: {type: String, enum: ['short', 'long'], optional: true},
    applicationUrl: String,
    shortApplicationUrl: {type: String, optional: true},
    logoUrl: {type: String, optional: true},
    useCustomBanner: Boolean,
    defaultBannerUrl: {type: String, optional: true},
    customBannerUrl: String,
    externalId: {type: String, index: true},
    legacyFinderId: {type: String, index: true},
    commission: {type: ObjectId, mongooseRef: 'Commission', index: true},
    timezone: String,
    naturalExpiry: Boolean,

    // Details for job postings managed by R2:
    mainImage: String,
    clientName: String,
    hideClientName: Boolean,
    clientDescription: String,
    molProfession: new Complex({
      code: String,
      profession: String,
      category: String,
      description: String
    }),
    visibilityPack: {type: String},
    visibilityPackVideoUrl: String,
    clientPresentationUrl: String,
    employmentDuration: {type: String, enum: ['LIMITED', 'CONTINUOUS', 'NOT_SPECIFIED', 'SUMMER']},
    workType: {type: String, enum: ['WHOLE_TIME', 'PART_TIME', 'HOURLY']},
    workDuration: {
      type: String,
      enum: ['LESS_THAN_11_DAYS', 'ELEVEN_DAYS_TO_ONE_MONTH', 'ONE_TO_THREE_MONTHS', 'THREE_TO_SIX_MONTHS', 'SIX_TO_TWELVE_MONTHS', 'OVER_12_MONTHS']
    },
    salaryPreference: new Complex({
      required: {type: Boolean},
      period: {type: String, enum: ['month', 'hour', 'year']},
      currency: {type: String, enum: ['EUR', 'SEK', 'USD']},
      freeform: {type: Boolean} // for job postings before urapalvelu
    }),
    jobIndustry: String,
    lead: String,
    jobStartDate: {type: Date, optional: true, allowNull: true},
    untilSuitableCandidateFound: Boolean,
    requirements: [String],
    legacyHtmlFormat: Boolean,
    expectations: [String],
    billableCompany: new Complex({
      businessId: String,
      companyName: String,
      companyNumber: String,
      industry: {type: String, optional: true}
    }),
    locations: [new Complex({
      communityname: {type: String, optional: true},
      communitycode: {type: String, optional: true},
      countrycode: {type: String, optional: true},
      postalcode: {type: String, optional: true},
      statename: {type: String, optional: true},
      custom: {type: String, optional: true}
    })],
    language: {type: String, mongooseDefault: 'en'},
    jobDetails: String,
    channels: new Complex({
      mol: new Complex({
        active: Boolean,
        trulyActive: Boolean,
        updatePending: {type: Date, optional: true},
        externalId: String
      }),
      barona: new Complex({
        active: Boolean,
        externalId: {type: String, index: true},
        pregeneratedExternalId: String
      }),
      monster: new Complex({
        active: Boolean,
        externalId: String
      }),
      oikotie: new Complex({
        active: Boolean,
        externalId: String
      }),
      urapalvelu: new Complex({
        active: Boolean,
        externalId: String
      })
    }),
    customChannels: [String],
    contactInformation: new Complex({
      contactName: String,
      contactEmail: String,
      contactPhone: String
    }),
    rbnIntegration: {optional: true,
      type: new Complex({
      rbnOneId: String,
      interviewName: String,
      endTime: Date,
      language: String,
      answerableQuestions: Number,
      questions: [new Complex({
        title: String,
        answerable: Boolean
      })]
    })},
    applicationSentMessage: String,
    applicationReceivedEmail: String,
    temp: Number,
    customerId: {type: String, index: true}
  }
}

export default {
  get mongoose() {
    return toMongooseSchema('jobPostingMongooseSchema', schema)
  },
  get typescript() {
    return toTypescriptSchema('JobPosting', schema, 'typescript', { omitExtraExports: true })
  },
  get json() {
    return toJSONSchema('ExampleJSONSchema', schema, false)
  }
}
