import {Schema} from '../schema'
import ObjectId from '../object-id'
import Complex from '../complex'
import toMongooseSchema from '../mongoose-output'
import toTypescriptSchema from '../ts-output'
import toJSONSchema from '../json-schema-output'
import {R2Address, R2Author} from './r2-common'
import R2Interview from './r2-interview'
import {R2AuthoredText} from './r2-authored-text'
import now from '../now'
import {R2Attachments} from './r2-attachment'
import {R2CommissionInfo} from './r2-commission-info'
import {R2Education} from './r2-education'
import {R2WorkExperience} from './r2-workexperience'
import {R2Comment} from './r2-comment'
import {R2Contract} from './r2-contract'
import {R2EmploymentContract} from './r2-employment-contract'
import {R2Language} from './r2-language'

const jobApplication = new Complex({
  _id: ObjectId,
  externalId: String,
  date: {type: Date, index: true},
  availability: String,
  additionalAvailabilityInfo: {type: String, optional: true},
  position: new Complex({
    name: String,
    businessBranch: {type: String, optional: true},
    city: {type: String, optional: true},
    externalId: {type: String, index: true},
    commission: {type: ObjectId, index: true}
  }),
  text: {type: String, optional: true},
  desiredSalary: {type: String, optional: true},
  salaryPreferenceAnswer: {
    optional: true,
    type: new Complex({
      amount: {type: Number, optional: true},
      period: {type: String, enum: ['month', 'hour', 'year'], optional: true},
      currency: {type: String, enum: ['EUR', 'SEK', 'USD'], optional: true},
      freeform: {type: String, optional: true}
    })
  },
  cover: {type: String, optional: true},
  attachments: [ObjectId],
  source: {type: String, optional: true},
  expectations: {
    optional: true,
    type: [new Complex({
      label: String,
      fulfilled: Boolean
    })]
  },
  requirements: {
    optional: true,
    type: [new Complex({
      label: String,
      fulfilled: Boolean
    })]
  }
  ,
  applicationForm: {type: ObjectId, mongooseRef: 'ApplicationForm', index: true, optional: true},
  answers: [new Complex({
    questionId: ObjectId,
    questionType: String,
    label: {type: String, optional: true},
    answer: Object
  })]
})

const schema: Schema = {
  fields: {
    _id: ObjectId,
    finderId: {type: Number, index: true},
    pendingDelete: {type: Date, index: true, optional: true},
    isDeleted: Boolean,
    undeleteToken: {type: String, index: 'unique-sparse', optional: true},
    backupInfo: Object,
    lastModified: {type: Date, mongooseDefault: now},
    createdBy: {type: ObjectId, mongooseRef: 'User'},
    externalId: {type: String, index: true, optional: true},
    rbnAtsId: String,
    lastDuplicateCheck: {type: Date, optional: true},
    mergedTo: ObjectId,
    potentialDuplicates: [new Complex({
      person: {type: ObjectId, index: true},
      resolution: {type: String, enum: ['none', 'merged', 'not-duplicate'], mongooseDefault: 'none'},
      resolvedBy: {type: R2Author, optional: true},
      resolvedAt: Date,
      mergeDetails: [
        new Complex({
          name: new Complex({last: String, first: String}),
          email: {type: String, optional: true},
          phone: {type: String, optional: true}
        })
      ]
    })],
    linkedInUrl: String,
    username: String,
    email: {type: String, index: true, optional: true},
    knownEmails: [String],
    phone: {type: String, optional: true},
    name: new Complex({
      first: String,
      last: String
    }),
    address: R2Address,
    nationality: String,
    expertise: [String],
    interviews: [R2Interview],
    overview: [new Complex(R2AuthoredText.fields)],
    introductionText: String,
    finder: new Complex({
      profileComment: String,
      education: String,
      additionalInformation: String,
      employeeWishes: String
    }),
    myProfile: new Complex({
      education: [R2Education],
      workExperience: [R2WorkExperience],
      jobApplications: [
        jobApplication
      ],
      languages: [
        R2Language
      ],
      languagesEditClone: new Complex({
        state: {type: String, enum: ['saved', 'editing'], mongooseDefault: 'saved', index: true},
        author: R2Author,
        date: {type: Date, mongooseDefault: now},
        languages: [R2Language]
      }),
      attachments: [R2Attachments],
      exists: Boolean
    }),
    ssn: {type: String, optional: true},
    sex: String, // 'm' || 'f'
    iban: String,
    taxNumber: Number,
    employmentContracts: [R2EmploymentContract],
    contracts: [R2Contract],
    comments: [R2Comment],
    rbnInterviews: [{type: ObjectId, mongooseRef: 'RbnInterview', index: true}],
    latestRbnInterview: {type: Date, optional: true},
    hidden: Boolean,
    interviewedByIndustries: [String],
    notEmployableInIndustries: [String],
    birthDate: {type: Date, optional: true},
    computedAvailableAt: {type: Date, optional: true},
    availableAt: {type: Date, optional: true},
    availableUntil: {type: Date, optional: true},
    timeNeeded: new Complex({
      amount: {type: Number, optional: true},
      unit: {type: String, optional: true}
    }),
    hasCar: {type: Boolean, optional: true},
    synced: Boolean,
    editClone: {
      optional: true,
      type: new Complex({
        exists: Boolean,
        email: {type: String, optional: true},
        phone: {type: String, optional: true},
        name: new Complex({first: String, last: String}),
        address: R2Address,
        ssn: {type: String, optional: true},
        sex: String,
        iban: String,
        taxNumber: String
      })
    },
    profileSearchKeys: [String],
    unsaved: Boolean,
    transferredToFinderBasket: {type: Date, optional: true},
    reservedBy: [
      new Complex({
        user: {type: ObjectId, mongooseRef: 'User'},
        name: new Complex({last: String, first: String}),
        date: {type: Date, mongooseDefault: now},
        checked: Boolean
      })
    ],
    commissionInfo: [
      R2CommissionInfo
    ],
    integrationPointId: Object,
    customerId: {type: String, index: true}
  }
}

export default {
  get mongoose() {
    return toMongooseSchema('personMongooseSchema', schema)
  },
  get typescript() {
    return toTypescriptSchema('Person', schema, 'typescript', {omitExtraExports: true})
  },
  get json() {
    return toJSONSchema('ExampleJSONSchema', schema, false)
  }
}
